unit DecisionFormUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Buttons, ComCtrls;

type
  TImageAction=(iaDelete1,iaDelete2,iaNone);     // Action choisie par l'utilisateur: supprimer la 1re image, la seconde, ou ne rien faire

  TDecisionForm = class(TForm)
    Panel1: TPanel;
    GroupBox1: TGroupBox;
    ScrollBox1: TScrollBox;
    PaintBox1: TPaintBox;
    GroupBox2: TGroupBox;
    ScrollBox2: TScrollBox;
    PaintBox2: TPaintBox;
    RadioGroup1: TRadioGroup;
    CheckBox1: TCheckBox;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    BitBtn3: TBitBtn;
    Panel2: TPanel;
    TrackBar1: TTrackBar;
    CheckBox2: TCheckBox;
    Panel3: TPanel;
    TrackBar2: TTrackBar;
    CheckBox3: TCheckBox;
    StaticText1: TEdit;
    StaticText2: TEdit;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Panel1Resize(Sender: TObject);
    procedure RadioGroup1Click(Sender: TObject);
    procedure PaintBox1Paint(Sender: TObject);
    procedure PaintBox2Paint(Sender: TObject);
    procedure CheckBox2Click(Sender: TObject);
  private
    FPicture1,FPicture2:TPicture;
  public
    Canceled:Boolean;

    function Execute(FileName1,FileName2:string;Similiraty:Single):TImageAction;     // Renvoie l'action choisie par l'utilisateur

    procedure ApplyZoom;
  end;

var
  DecisionForm: TDecisionForm;

implementation

uses
  MainFormUnit;

{$R *.dfm}

procedure TDecisionForm.FormCreate(Sender: TObject);
begin
  FPicture1:=TPicture.Create;
  FPicture2:=TPicture.Create;
  CheckBox2.Align:=alRight;
  CheckBox2.Left:=1000;
  CheckBox3.Align:=alRight;
  CheckBox3.Left:=1000;
  StaticText1.Align:=alBottom;
  StaticText2.Align:=alBottom;
end;

procedure TDecisionForm.FormDestroy(Sender: TObject);
begin
  FPicture1.Destroy;
  FPicture2.Destroy;
end;

procedure TDecisionForm.Panel1Resize(Sender: TObject);
begin
  GroupBox1.Width:=Panel1.ClientWidth div 2;
  ApplyZoom;
end;

procedure TDecisionForm.RadioGroup1Click(Sender: TObject);
begin
  BitBtn1.Enabled:=RadioGroup1.ItemIndex>-1;  // Il faut dj prendre une dcision avant de pouvoir la confirmer
end;

procedure TDecisionForm.PaintBox1Paint(Sender: TObject);
begin
  PaintBox1.Canvas.StretchDraw(PaintBox1.ClientRect,FPicture1.Graphic);  // Affichage de l'image 1
end;

procedure TDecisionForm.PaintBox2Paint(Sender: TObject);
begin
  PaintBox2.Canvas.StretchDraw(PaintBox2.ClientRect,FPicture2.Graphic);  // Affichage de l'image 1
end;

function GetFileSize(FileName:string):Int64;   // Donne la taille d'un fichier
var
  f:TSearchRec;
begin
  if FindFirst(FileName,faAnyFile,f)=0 then
    try
      {$WARNINGS OFF}
      Result:=Int64(f.FindData.nFileSizeHigh) shl 32+f.FindData.nFileSizeLow;
      {$WARNINGS ON}
    finally
      FindClose(f);
    end
  else
    Result:=0;
end;

function TDecisionForm.Execute(FileName1, FileName2: string;
  Similiraty: Single): TImageAction;
var
  m:TModalResult;
const
  T:array[False..True] of TImageAction=(iaDelete1,iaDelete2);
begin
  Result:=iaNone;
  if CheckBox1.Checked then // Si l'utilisateur a dcid de toujours faire la mme action
    m:=mrOk                 // on ne lui demande pas son avis de nouveau
  else begin
    Caption:=Format('Similarity: %f %%',[Similiraty]);
    FPicture1.LoadFromFile(FileName1);
    FPicture2.LoadFromFile(FileName2);
    StaticText1.Text:='.\'+Copy(FileName1,Length(MainForm.Edit1.Text)+2,Length(FileName1));
    StaticText2.Text:='.\'+Copy(FileName2,Length(MainForm.Edit1.Text)+2,Length(FileName2));
    PaintBox1.Hint:=Format('Filename: "%s"'#13'Dimensions: %d x %d'#13'Date: '+DateTimeToStr(FileDateToDateTime(FileAge(FileName1))),[FileName1,FPicture1.Width,FPicture1.Height]);
    PaintBox2.Hint:=Format('Filename: "%s"'#13'Dimensions: %d x %d'#13'Date: '+DateTimeToStr(FileDateToDateTime(FileAge(FileName2))),[FileName2,FPicture2.Width,FPicture2.Height]);
    ScrollBox1.Hint:=PaintBox1.Hint;
    ScrollBox2.Hint:=PaintBox2.Hint;
    GroupBox1.Caption:=Format('Image 1 : %d x %d',[FPicture1.Width,FPicture1.Height]);
    GroupBox2.Caption:=Format('Image 2 : %d x %d',[FPicture2.Width,FPicture2.Height]);
    ApplyZoom;
    m:=ShowModal;           // on demande son avis  l'utilisateur
  end;
  case m of
    mrOk:begin
      case RadioGroup1.ItemIndex of
        0,4:Result:=T[RadioGroup1.ItemIndex=4];
        1,5:Result:=T[(FPicture1.Width*FPicture1.Height<FPicture2.Width*FPicture2.Height) xor (RadioGroup1.ItemIndex=1)];
        2,6:Result:=T[(GetFileSize(FileName1)<GetFileSize(FileName2)) xor (RadioGroup1.ItemIndex=2)];
        3,7:Result:=T[(FileDateToDateTime(FileAge(FileName1))<FileDateToDateTime(FileAge(FileName2))) xor (RadioGroup1.ItemIndex=3)];
      end;
    end;
    mrCancel:Canceled:=True;  // Opration annule
  else
    CheckBox1.Checked:=False; // Si l'utilisateur n'a pas pris de dcision, alors on lui demandera son avis la prochaine fois
  end;
end;

procedure TDecisionForm.ApplyZoom;

  procedure Update(Picture:TPicture;PaintBox:TPaintBox;ScrollBox:TScrollBox;CheckBox:TCheckBox;Panel:TPanel;TrackBar:TTrackBar);
  const
    U:array[-4..4] of integer=(1,1,1,1,1,2,4,8,16);
    V:array[-4..4] of integer=(16,8,4,2,1,1,1,1,1);
  var
    a,b:Integer;
  begin
    with Picture do begin
      PaintBox.Parent:=nil;
      if CheckBox.Checked then begin
        a:=ScrollBox.ClientWidth;
        b:=ScrollBox.ClientHeight;
        Panel.Caption:='Zoom';
        TrackBar.Visible:=False;
        PaintBox.Parent:=ScrollBox;
        if Width*b>a*Height then
          PaintBox.SetBounds(0,(b*Width-a*Height) div (2*Width),a,(a*Height) div Width)
        else
          PaintBox.SetBounds((a*Height-b*Width) div (2*Height),0,(b*Width) div Height,b);
      end else begin
        a:=U[TrackBar.Position];
        b:=V[TrackBar.Position];
        Panel.Caption:='Zoom level: '+IntToStr(a)+':'+IntToStr(b);
        TrackBar.Visible:=True;
        PaintBox.Parent:=ScrollBox;
        PaintBox.SetBounds(0,0,(Width*a) div b,(Height*a) div b);
      end;
    end;

  end;

begin
  Update(FPicture1,PaintBox1,ScrollBox1,CheckBox2,Panel2,TrackBar1);
  Update(FPicture2,PaintBox2,ScrollBox2,CheckBox3,Panel3,TrackBar2);
end;

procedure TDecisionForm.CheckBox2Click(Sender: TObject);
begin
  ApplyZoom;
end;

end.
